<div class="radio">
    <label>
        <input id="${field.id}" name="${field.name}" type="radio" {% if str:field.value == field.on %}checked="checked"{% endif %} value="${field.on}" {% if field.required %}required{% endif %}{% if field.disabled %} disabled{% endif %}/> ${field.text}
    </label>
</div>