"""

An experimental work in progress asynchronous logic executor


"""

import threading


class ExecutorThread(threading.thread):
    def __init__(self):
        pass


class AsyncLogic(object):
    pass
