{%- for field in self.children %}
{%- if field.is_form_field and field.visible %}
    <div class="form-group{% if field.errors %} has-error{% endif %}">
        {%- if field.label %}
        <label {% attrib class=[field.hidelabel ? 'sr-only' : None] %} for="${field.id}">${field.label}</label>
        {%- endif %}
        {%- render field set style="simple" %}
        {%- include "common/helperror.html" from "moya.forms" %}
    </div>
{%- else %}
    {%- render field set style="simple" %}
{%- endif %}
{% endfor %}
