<input id="${field.id}" name="${field.name}" type="password" value="${field.value or ''}" class="form-control ${field.class}" {% if field.placeholder %} placeholder="${field.placeholder}"{% endif %} {%if field.required %}required{% endif %}{% if field.disabled %} disabled{% endif %}/>
